#!/usr/bin/env python3
import json
import os
import subprocess
import uuid
from collections import OrderedDict

import falcon
from oslo_log import log
from tveapi.common import resource as res
import sys
sys.path.insert(1, '/usr/lib/python3/dist-packages/tveapi/')
import send_notification
LOG = log.getLogger(__name__)


class Probe(object):
    @staticmethod
    def on_fail(resp, message):
        errmsg = OrderedDict()
        errmsg['404'] = ("{}".format(message))
        resp.body = errmsg
        resp.status = falcon.HTTP_404
        resp.reason = json.dumps(errmsg, indent=4, separators=(',', ': ')).encode('utf8')
        resp.content_type = 'application/json'

    class Onboard(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Probe.on_fail(resp, 'No application')
                return

            appname = json_req['application']
            if not os.path.exists('/home/admin/user_files/{}.tar.gz'.format(appname)):
                Probe.on_fail(resp, 'No {}.tar.gz'.format(appname))
                return

            ret = subprocess.call(
                "/usr/bin/sudo /scripts/application_onboard.sh {}".format(appname),
                shell=True)
            if ret == 3:
                resp.status = falcon.HTTP_403
                Probe.on_fail(resp, 'Native applications are disabled or not allowed by a valid license.')
                return
            elif ret != 0:
                Probe.on_fail(resp, 'Onboarding {} fail'.format(appname))
                return

            resp.body = {'onboard': 'ok'}
            resp.status = falcon.HTTP_200
            return

    class Remove(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Probe.on_fail(resp, 'No application')
                return

            appname = json_req['application']
            if 'clavister' in appname.lower():
                subprocess.call(['/usr/bin/sudo', 'touch', '/run/.reboot_in_progress'])

            ret1 = subprocess.call("/usr/bin/sudo systemctl disable {}.service".format(
                appname.lower()), shell=True)
            ret2 = subprocess.call("/usr/bin/sudo systemctl --no-block stop {}.service".format(
                appname.lower()), shell=True)
            ret3 = subprocess.call("/usr/bin/sudo rm -rf /root/apps/{}".format(
                appname.lower()), shell=True)
            ret4 = subprocess.call("/usr/bin/sudo systemctl reset-failed {}.service || true".format(
                appname.lower()), shell=True)
            ret5 = subprocess.call("/usr/bin/sudo rm -f /lib/systemd/system/{}.service".format(
                appname.lower()), shell=True)
            ret6 = subprocess.call("/usr/bin/sudo systemctl daemon-reload",
                                   shell=True)
            if ret1 != 0 or ret2 != 0 or ret3 != 0 or ret4 != 0 or ret5 != 0 or ret6 != 0:
                Probe.on_fail(resp, 'Removing {} fail'.format(appname.lower()))
                return

            resp.body = {'remove': 'ok'}
            resp.status = falcon.HTTP_200
            if 'clavister' in appname.lower():
                if os.path.exists('/config/_reboot/tve/now'):
                    subprocess.call(['/usr/bin/sudo', 'rmdir', '/config/_reboot/tve/now'])

                subprocess.call(['/usr/bin/sudo', 'mkdir', '/config/_reboot/tve/now'])

            return

    class Start(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Probe.on_fail(resp, 'No application')
                return

            appname = json_req['application']

            ret = subprocess.call(
                "/usr/bin/sudo /scripts/application_start.sh {}".format(appname.lower()),
                shell=True)
            if ret == 3:
                resp.status = falcon.HTTP_403
                Probe.on_fail(resp, 'Native applications are disabled or not allowed by a valid license.')
                return
            elif ret != 0:
                Probe.on_fail(resp, 'Starting {} fail'.format(appname.lower()))
                return

            resp.body = {'start': 'ok'}
            resp.status = falcon.HTTP_200
            return

    class Stop(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Probe.on_fail(resp, 'No application')
                return

            appname = json_req['application']

            ret = subprocess.call(
                "/usr/bin/sudo /scripts/application_stop.sh {}".format(appname.lower()),
                shell=True)
            if ret == 3:
                Probe.on_fail(resp, 'Native applications are disabled or not allowed by a valid license.')
                return            
            elif ret != 0:
                Probe.on_fail(resp, 'Stoping {} fail'.format(appname.lower()))
                return

            resp.body = {'stop': 'ok'}
            resp.status = falcon.HTTP_200
            return

    class List(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            # listdir - No Permissions
            auuid = uuid.uuid4()
            if not req.content_length:
                try:
                    ret = subprocess.call(
                        '/usr/bin/sudo /scripts/application_list.sh all {}'.format(auuid),
                        shell=True)
                    if ret == 3:
                        Probe.on_fail(resp, 'Native applications are disabled or not allowed by a valid license.')
                        return                    
                    elif ret != 0:
                        Probe.on_fail(resp, 'List fail!')
                        return
                except Exception as e:
                    LOG.error('List fail!')
                    Probe.on_fail(resp, 'List fail!')
                    return

                try:
                    with open('/tmp/application_list_{}.json'.format(auuid), 'r') as json_file:
                        data = json.load(json_file)

                except Exception as e:
                    LOG.error('Reading application_list.json failed !')
                    Probe.on_fail(resp, 'List fail')
                    return

                resp.body = data
                resp.status = falcon.HTTP_200
                return

            json_req = self.json_body(req)
            if 'application' not in json_req:
                Probe.on_fail(resp, 'No application')
                return

            appname = json_req['application']
            ret = subprocess.call(
                '/usr/bin/sudo /scripts/application_list.sh {} {}'.format(appname.lower(), auuid),
                shell=True)
            if ret == 3:
                Probe.on_fail(resp, 'Native applications are disabled or not allowed by a valid license.')
                return
            elif ret != 0:
                Probe.on_fail(resp, 'List {} fail'.format(appname.lower()))
                return

            try:
                with open('/tmp/application_list_{}.json'.format(auuid), 'r') as json_file:
                    data = json.load(json_file)

            except Exception as e:
                LOG.error('Reading application_list.json failed !')
                Probe.on_fail(resp, 'List {} fail'.format(appname.lower()))
                return

            try:
                ret = subprocess.call(
                    '/usr/bin/sudo rm -f /tmp/application_list_{}.json'.format(auuid),
                    shell=True)
            except Exception as e:
                LOG.error('List fail!')
                Probe.on_fail(resp, 'List fail!')
                return

            resp.body = data
            resp.status = falcon.HTTP_200
            return

    class Retrieve(res.BaseResource):
        @falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Probe.on_fail(resp, 'No application')
                return

            appname = json_req['application']
            ret = subprocess.call(
                "/usr/bin/sudo ls /root/apps/{}".format(appname.lower()),
                shell=True)
            if ret != 0:
                Probe.on_fail(resp, 'Missing {}'.format(appname.lower()))
                return

            ret = subprocess.call(
                "/usr/bin/sudo /scripts/application_retrieve.sh {}".format(appname.lower()),
                shell=True)
            if ret == 3:
                Probe.on_fail(resp, 'Native applications are disabled or not allowed by a valid license.')
                return            
            elif ret != 0:
                Probe.on_fail(resp, 'Retrieve {} fail'.format(appname.lower()))
                return

            resp.body = {'retrieve': 'ok'}
            resp.status = falcon.HTTP_200
            return

    class RetrieveConfigs(res.BaseResource):
        #@falcon.before(send_notification.request_notifier)
        def on_put(self, req, resp):
            json_req = self.json_body(req)
            if 'application' not in json_req:
                Probe.on_fail(resp, 'No application')
                return

            appname = json_req['application']
            ret = subprocess.call(
                "/usr/bin/sudo ls /root/apps/{}".format(appname.lower()),
                shell=True)
            if ret != 0:
                Probe.on_fail(resp, 'Missing {}'.format(appname.lower()))
                return

            ret = subprocess.call(
                "/usr/bin/sudo /scripts/application_retrieveconfigs.sh {}".format(appname.lower()),
                shell=True)
            if ret == 3:
                Probe.on_fail(resp, 'Native applications are disabled or not allowed by a valid license.')
                return
            elif ret != 0:
                Probe.on_fail(resp, 'RetrieveConfigs {} fail'.format(appname.lower()))
                return

            resp.body = {'retrieveconfigs': 'ok'}
            resp.status = falcon.HTTP_200
            return
